/************************************************************************/
/*  asc2ms - Convert qmerge ASCII files to MiniSEED file.		*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: asc2ms.c,v 1.2 2000/10/26 12:13:29 doug Exp $ ";
#endif

#include <stdio.h>

#include "version.h"

#ifndef	DEFAULT_NETWORK
#define	DEFAULT_NETWORK		"XX"
#endif

#ifndef	DEFAULT_LOCATION
#define	DEFAULT_LOCATION	""
#endif

#ifndef	DEFAULT_OUTPUT_FORMAT
#define	DEFAULT_OUTPUT_FORMAT	STEIM1
#endif

#ifndef	DEFAULT_BLKSIZE
#define	DEFAULT_BLKSIZE		4096
#endif

#ifndef	DEFAULT_DATA_PER_LINE
#define	DEFAULT_DATA_PER_LINE 8
#endif

#define	QUOTE(x)	#x
#define	STRING(x)	QUOTE(x)

char *syntax[] = {
"%s version " VERSION,
"%s  [-S station] [-C channel] [-N network] [-L location] [-l 0|1] [-d n]",
"    [-b blksize] [-F format] [-h] [infile [outfile]]",
"    where:",
"	-S station  Specify explicit station name.",
"	-C channel  Specify explicit channel name.",
"	-N network  Specify explicit network name.  Default network name is " 
		    DEFAULT_NETWORK ".",
"	-L location Specify explicit location name.  Default location name is " 
		    DEFAULT_LOCATION ".",
"	-d n	    Specifies n data values per line (5 or 8).",
"		    Default is " STRING(DEFAULT_DATA_PER_LINE) ".",
"	-b blksize  Specifies output blksize.  Default blocksize=" STRING(DEFAULT_BLKSIZE) ".",
"	-F format   Specifies output format for MiniSEED data.",
"		    Default output format is " STRING(DEFAULT_OUTPUT_FORMAT) ".",
"	-h	    Help - prints syntax message.",
"	infile	    Input file containing Quanterra ASCII data.",
"		    If no input file is specified, data is read from stdin.",
"	outfile	    Output file for MiniSEED Records.",
"		    If no output file is specified, output is written to stdout.",
NULL };

#include "qlib2.h"

#define	EXTERN
#define	DEFINE

#include "externals.h"
#include "procs.h"

/************************************************************************/
/*  main:   main program.						*/
/************************************************************************/
main (int argc, char **argv)
{
    int i, ival, npts;
    DATA_HDR *hdr;
    int *data = NULL;		/* allow qlib to allocate buffer.	*/
    char *mseed = NULL;		/* allow qlib to allocate buffer.	*/
    char *p;
    int n, nleft;
    int status;
    int nblocks;

    info = stdout;
    parse_cmdline(&argc,&argv);
    if ( (hdr = read_hdr(input)) == NULL) {
	fprintf (stderr, "Error reading header in ascii file %s\n", infile);
	exit(1);
    }
    /* Provide defaults if the file had no entries. */
    if (strlen(hdr->network_id) == 0) capnstr(hdr->network_id,DEFAULT_NETWORK,2);
    if (strlen(hdr->location_id) == 0) capnstr(hdr->location_id,DEFAULT_LOCATION,2);
    /* Override if specified on command line. */
    if (strlen(station)>0) capnstr(hdr->station_id,station,5);
    if (strlen(channel)>0) capnstr(hdr->channel_id,channel,3);
    if (strlen(network)>0) capnstr(hdr->network_id,network,2);
    if (strlen(location)>0) capnstr(hdr->location_id,location,2);

    npts = read_data (&data, input);
    hdr->data_type = output_format;
    hdr->blksize = blksize;
    status = ms_pack_data (hdr, NULL, npts, data, &nblocks, &mseed, 0, NULL);
    if (status != npts) FATAL("Error packing data into MiniSEED.")

    /* Write MiniSEED data to output file.				*/
    n = nleft = nblocks;
    p = (char *)mseed;
    while (nleft > 0 && n > 0) {
	n = fwrite (p, blksize, nleft, output);
	p += n*blksize;
	nleft -= n;
    }
    if (nleft) FATAL("Error writing MiniSEED data.");

    free_data_hdr (hdr);
    free (data);
    free (mseed);
    fclose (input);
    fclose (output);
    return(0);
}
