/************************************************************************/
/*  Routines to read input data for asc2ms.				*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: read_data.c,v 1.2 2000/10/26 12:13:29 doug Exp $ ";
#endif

#include <stdio.h>
#include <malloc.h>

#include <qlib2.h>

#include "procs.h"
#include "externals.h"

#define MSIZE	250000
/************************************************************************/
/*  read_data:								*/
/*	Read specified number of ascii data values from the input file. */
/*  return:								*/
/*	number of samples read.						*/
/************************************************************************/
int read_data 
   (int		**pdata,	/* ptr to ptr to input buffer.		*/
    FILE	*input)		/* input FILE.				*/
{
    int *data = NULL;
    char line[256];
    int nalloc = 0;
    int n = 0;

    if ((data = (int *)malloc(MSIZE*sizeof(int))) == NULL) {
	return (0);
    }
    nalloc += MSIZE;

    n = 0;
    while (fgets(line, 256, input)) {
	if (line[0] == 's') continue;
	if (n+1 > nalloc) {
	    if ((data = (int *)realloc(data, (nalloc+MSIZE)*sizeof(int))) == NULL);
	    return (0);
	}
	nalloc += MSIZE;
	data[n++] = atoi (line);
    }
    *pdata = data;
    return (n);
}
